/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: %W%
//  Last update: %E% %U%
//  SCCS path, id: %F% %I%
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*************************************************************************
//
//  Licensed Materials - Property of IBM
//  WA81 SupportPac
//  (c) Copyright IBM Corp. 1999, 2001
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//************************************************************************/
/*END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.servlet.client;

import java.util.*;
import java.io.File;
import javax.servlet.http.HttpServletRequest;

import com.ibm.workflow.api.*;

/**
* <p>This Viewer should be used when your Application server supports the
* JSP V1.0 standard. It uses JSPs for custom process and activity forms as
* well as for some other commands. This provides much more flexibility than
* the propriatary HTML template files used by <code>DefaultViewer</code> and
* it also supports internationalization.</p>
* <p>A lot of control flow logic is inherited from the base class
* <code>DefaultViewer</code>, here only those methods that actually invoke
* JSPs are overridden.</p>
* <p>You must set
* <code>DefaultViewer=com.ibm.worklflow.servlet.client.JSPViewer</code> in the
* <a href="../../../../../../properties.html#Viewer">WebClient.properties</a>
* file to in order to enable this <code>Viewer</code>.</p>
*/
public class JSPViewer extends DefaultViewer
{
  /**
  * Creates the page that is shown in case of an error. This method implements
  * 'logon splicing'. Errors other than {@link NoSessionException} are
  * passed on to the base class. In the case of an expired session, the
  * original command will be remembered and executed after a successful
  * re-logon.
  * @param context The request context.
  * @return The response page.
  */
  public ResponsePage errorResponse(RequestContext context)
  {
    ClientException error = context.getException();
    HttpServletRequest request = error.getRequest();
    String command = request == null ? null : request.getParameter("command");

    // only handle NoSessionException here
    if (command == null || !(error instanceof NoSessionException))
      return super.errorResponse(context);

    // doing a re-logon just for logging of? Dunno...
    if ("logoff".equals(command))
      return logoffResponse(context);

    LogonContext ctx = new LogonContext(context);
    ctx.setMessage(context.get("Message.ReLogon"));
    ctx.addParameter("command2", command);

    Enumeration e = request.getParameterNames();

    while (e.hasMoreElements())
    {
      String parm = (String) e.nextElement();

      if (!"command".equals(parm))
        ctx.addParameter(parm, request.getParameter(parm));
    }

    return new ResponsePage(ctx, "/Logon.jsp", -1);
  }

  /**
  * Creates the page that is displayed after a successful logon.
  * If this was a spliced logon page (see {@link #errorResponse}), the
  * remembered command is extracted and executed. Otherwise, the base
  * class is called to display the first list. If no list is found, the
  * page to create a new list will be returned.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage logonResponse(RequestContext context)
                      throws ClientException
  {
    BuiltinHandler builtin = context.getBuiltinHandler();

    // fill the list caches
    builtin.queryProcessTemplateLists(context);
    builtin.queryProcessInstanceLists(context);
    builtin.queryWorkLists(context);

    HttpServletRequest request = context.getRequest();
    String command = request == null ? null : request.getParameter("command2");

    if (command != null)
      return builtin.execute(command, request, null);

    ResponsePage result = mostRecentList(context, false);

    if (result == null) // no lists available
      result = queryListsResponse(context);

    return result;
  }

  /**
  * Creates the page that is shown after logoff.
  * This request will be forwarded to <code>Logon.jsp</code>.
  * @param context The request context.
  * @return The response page.
  */
  public ResponsePage logoffResponse(RequestContext context)
  {
    return new ResponsePage(new LogonContext(context), "/Logon.jsp", -1);
  }

  /**
  * Creates the page that displays the list of lists.
  * This request will be forwarded to <code>forms/ListViewer.jsp</code>
  * which dynamically creates a page that contains a table with a row
  * for each list on the list. This table also contains buttons that
  * allow to invoke other commands, such as deleting a list.
  * @param context The request context.
  * @return The response page or <code>null</code> if lists of lists are
  *         disabled
  * @exception ClientException if an error occurred
  */
  public ResponsePage queryListsResponse(RequestContext context)
                      throws ClientException
  {
    if (!enableLists)
      return null;

    context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

    return new ResponsePage(context, "/forms/ListViewer.jsp");
  }

  /**
  * Creates the page that displays list settings.
  * This request will be forwarded to <code>forms/ListPropertiesViewer.jsp</code>.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage showListPropertiesResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/ListPropertiesViewer.jsp");
  }

  /**
  * Creates the page that displays the list of lists.
  * This request will be forwarded to <code>forms/ListViewer.jsp</code>
  * which dynamically creates a page that contains a table with a row
  * for each list.
  * @param context The request context.
  * @return The response page or <code>null</code> if template lists are
  *         disabled
  * @exception ClientException if an error occurred
  */
  public ResponsePage setListPropertiesResponse(RequestContext context)
                      throws ClientException
  {
    context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));
    return new ResponsePage(context, "/forms/ListViewer.jsp");
  }

  /**
  * @doc
  */
  public ResponsePage createListResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/ListPropertiesViewer.jsp");
  }

  /**
  * Creates the page that displays a process template list.
  * This request will be forwarded to <code>forms/ListViewer.jsp</code>
  * which dynamically creates a page that contains a table with a row
  * for each template on the list. This table also contains buttons that
  * allow to invoke other commands, such as displaying a template's properties.
  * @param context The request context.
  * @return The response page or <code>null</code> if template lists are
  *         disabled
  * @exception ClientException if an error occurred
  */
  public ResponsePage queryProcessTemplatesResponse(RequestContext context)
                      throws ClientException
  {
    if (!enableTemplatelists)
      return null;

    context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

    return new ResponsePage(context, "/forms/ListViewer.jsp");
  }

  /**
  * Creates the page that displays process template properties.
  * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage showTemplatePropertiesResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
  }

  /**
  * Creates the page displayed after a new process instance has been created
  * and started.
  * <p>If the process instance was not yet started because it
  * <a href="BuiltinHandler.html#data_reqd">required input data</a>, the
  * request will be forwarded to <code>processes/<i>TemplateName</i>.jsp</code>.
  * This JSP should contain a form to fill in the required input container
  * data and then again call {@link BuiltinHandler#createAndStartInstance}.
  * For a sample, see the <a href="../../../../../../samples.html#credit">
  * credit request process</a>. If the JSP file could not be found, the
  * request will be passed to the base class <code>DefaultViewer</code>.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage createAndStartInstanceResponse(RequestContext context)
                      throws ClientException
  {
    try
    {
      if (context.getInstance() == null)
      {
        // not yet started because input data required
        String templateName = context.getTemplate().name();
        String urlName = "/processes/" + templateName + ".jsp";

        File jspFile = new File(context.getDocumentRoot(), urlName);

        if (jspFile.isFile())
          return new ResponsePage(context, urlName);

        context.log(jspFile + " not found, trying DefaultViewer", false);
      }
      else if (context.getRequestParameter("id", null) == null)
      {
        // process was started by the webuser
        String urlname = context.getRequestParameter("SuccessLocation", null);

        if (urlname != null && urlname.endsWith(".jsp"))
        {
          // JSPs should not be treated as HTML templates like
          // DefaultViewer does
          return new ResponsePage(context, context.getURL(urlname));
        }
      }

      return super.createAndStartInstanceResponse(context);
    }
    catch (FmcException xcpt)
    {
      throw new ClientException(context, xcpt);
    }
  }

  /**
  * Creates the page that displays a process instance list.
  * This request will be forwarded to <code>forms/ListViewer.jsp</code>
  * which dynamically creates a page that contains a table with a row
  * for each instance on the list. This table also contains buttons that
  * allow to invoke other commands, such as displaying an instance's properties.
  * @param context The request context.
  * @return The response page or <code>null</code> if template lists are
  *         disabled
  * @exception ClientException if an error occurred
  */
  public ResponsePage queryProcessInstancesResponse(RequestContext context)
                      throws ClientException
  {
    if (!enableInstancelists)
      return null;

    context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

    return new ResponsePage(context, "/forms/ListViewer.jsp");
  }

  /**
  * Creates the page that displays process instance properties.
  * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage showInstancePropertiesResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
  }

  /**
  * Creates the page that will be displayed after a process instance has
  * been started.
  * If the process instance was actually started, base class'
  * {@link DefaultViewer#startInstanceResponse startInstanceResponse} method
  * will be called. If the process instance was not yet started because it
  * required input data, the request will be forwarded to
  * <code>processes/<i>TemplateName</i>.jsp</code>, if this file exists.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage startInstanceResponse(RequestContext context)
                      throws ClientException
  {
    try
    {
      ProcessInstance instance = context.getInstance();

      if (instance.state().value() ==
          com.ibm.workflow.api.ProcessInstancePackage.ExecutionState._READY)
      {
        // not yet started because input data required
        String templateName = instance.processTemplateName();
        String urlName = "/processes/" + templateName + ".jsp";

        File jspFile = new File(context.getDocumentRoot(), urlName);

        if (jspFile.isFile())
          return new ResponsePage(context, urlName);

        context.log(jspFile + " not found, trying DefaultViewer", false);
      }

      return super.startInstanceResponse(context);
    }
    catch (FmcException xcpt)
    {
      throw new ClientException(context, xcpt);
    }
  }

  /**
  * Creates the page that is displayed after a process instance has been
  * suspended. If the process instance was actually suspended, the base
  * class' {@link DefaultViewer#suspendInstanceResponse suspendInstanceResponse}
  * method will be called. If it has not yet been suspended because the
  * necessary data has not been specified, the request will be forwarded
  * to <code>forms/SuspendInstance.jsp</code> so the parameters for the
  * {@link BuiltinHandler#suspendInstance suspendInstance} command can be
  * specified.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage suspendInstanceResponse(RequestContext context)
                      throws ClientException
  {
    if (context.getRequestParameter("submitData", null) != null)
      return super.suspendInstanceResponse(context);

    return new ResponsePage(context, "/forms/SuspendInstance.jsp");
  }

  /**
  * Creates the page that displays activity instance properties.
  * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage showActivityPropertiesResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
  }

  /**
  * Creates the page that is displayed after an activity instance has been
  * force restarted. If the activity instance was actually force restarted, the
  * request will be forwarded to <code>forms/PropertiesViewer.jsp</code> (this
  * is the only page from where activities can be force restarted). If it has
  * not yet been force restarted because the necessary data has not been
  * specified, the request will be forwarded to
  * <code>forms/ForceRestart.jsp</code> so the parameters for the
  * {@link BuiltinHandler#forceRestartActivity forceRestartActivity} command
  * can be specified.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage forceRestartActivityResponse(RequestContext context)
                      throws ClientException
  {
    if (context.getRequestParameter("submitData", null) == null)
      return new ResponsePage(context, "/forms/ForceRestart.jsp");

    return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
  }

  /**
  * Creates the page that is displayed after an activity instance has been
  * force finished. If the activity instance was actually force finished, the
  * request will be forwarded to <code>forms/PropertiesViewer.jsp</code> (this
  * is the only page from where activities can be force finished). If it has
  * not yet been force finished because the necessary data has not been
  * specified, the request will be forwarded to
  * <code>forms/ForceFinish.jsp</code> so the parameters for the
  * {@link BuiltinHandler#forceFinishActivity forceFinishActivity} command
  * can be specified.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage forceFinishActivityResponse(RequestContext context)
                      throws ClientException
  {
    if (context.getRequestParameter("submitData", null) == null)
      return new ResponsePage(context, "/forms/ForceFinish.jsp");

    return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
  }

  /**
  * Creates the page that is displayed after an activity instance has been
  * terminated. This request will be forwarded to
  * <code>forms/PropertiesViewer.jsp</code> (this is the only page from where
  * activities can be terminated).
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage terminateActivityResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
  }

  /**
  * Creates the page that displays control connector properties.
  * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage showControlConnectorPropertiesResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
  }

  /**
  * Creates the page that displays a worklist.
  * This request will be forwarded to <code>forms/ListViewer.jsp</code>
  * which dynamically creates a page that contains a table with a row
  * for each work item on the list. This table also contains buttons that
  * allow to invoke other commands, such as displaying an item's properties.
  * @param context The request context.
  * @return The response page or <code>null</code> if template lists are
  *         disabled
  * @exception ClientException if an error occurred
  */
  public ResponsePage queryWorkItemsResponse(RequestContext context)
                      throws ClientException
  {
    if (!enableWorklists)
      return null;

    context.setAttribute("navigatorEntries", createNavigatorEntries(context, false));

    return new ResponsePage(context, "/forms/ListViewer.jsp");
  }

  /**
  * Creates the page that displays work item properties.
  * This request will be forwarded to <code>forms/PropertiesViewer.jsp</code>.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage showWorkItemPropertiesResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/PropertiesViewer.jsp");
  }

  /**
  * Creates the page that will be displayed when a work item has been
  * checked out.
  * <p>This request will be forwarded to
  * <code>programs/<i>ProgramName</i>.html</code> (where <i>ProgramName</i>
  * is the name of the program assigned to the activity). This JSP should
  * contain a form to fill in the required input container
  * data and then call {@link BuiltinHandler#checkInWorkItem}.
  * For a sample, see the <a href="../../../../../../samples.html#credit">
  * credit request process</a>. If the JSP file could not be found, the
  * request will be passed to the base class <code>DefaultViewer</code>.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage checkOutWorkItemResponse(RequestContext context)
                      throws ClientException
  {
    try
    {
      WorkItem workItem = context.getWorkItem();

      String urlName = "/programs/" + workItem.implementation() + ".jsp";

      File jspFile = new File(context.getDocumentRoot(), urlName);

      if (jspFile.isFile())
      {
        // EF: Netscape reloads pages with immediate expiry even when only
        // the browser window is resized or printed. The idea was to set
        // the page expiry to 5 minutes for work items to avoid those
        // annoying 'already checked out' exceptions (note that this browser
        // behavior is not a problem for most other requests like showing the
        // worklist, ...). But this workaround is no good when the checkout
        // is cancelled. Because then the next 'checkout' request is satisfied
        // from the browser cache without actually checking out the work item -
        // resulting in a 'state does not allow this action' exception when
        // trying to check in the work item...

        // Date date = new Date(System.currentTimeMillis() + 300 * 1000);
        return new ResponsePage(context, urlName/*, date.getTime()*/);
      }

      context.log(jspFile + " not found, trying DefaultViewer", false);

      return super.checkOutWorkItemResponse(context);
    }
    catch (FmcException xcpt)
    {
      throw new ClientException(context, xcpt);
    }
  }

  /**
  * Creates the page that is displayed after a work item has been
  * force finished. If the work item was actually force finished, the
  * request will be forwarded to the base class'
  * {@link DefaultViewer#forceFinishWorkItemResponse forceFinishWorkItemResponse}
  * method. If it has not yet been force finished because the necessary data
  * has not been specified, the request will be forwarded to
  * <code>forms/ForceFinish.jsp</code> so the parameters for the
  * {@link BuiltinHandler#forceFinishWorkItem forceFinishWorkItem} command
  * can be specified.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage forceFinishWorkItemResponse(RequestContext context)
                      throws ClientException
  {
    WorkItem workItem = context.getWorkItem();

    if (context.getRequestParameter("submitData", null) == null)
      return new ResponsePage(context, "/forms/ForceFinish.jsp");

    return super.forceFinishWorkItemResponse(context);
  }

  /**
  * Creates the page that is displayed after a work item has been
  * force restarted. If the work item was actually force restarted, the
  * request will be forwarded to the base class'
  * {@link DefaultViewer#forceRestartWorkItemResponse forceRestartWorkItemResponse}
  * method. If it has not yet been force restarted because the necessary data
  * has not been specified, the request will be forwarded to
  * <code>forms/ForceRestart.jsp</code> so the parameters for the
  * {@link BuiltinHandler#forceRestartWorkItem forceRestartWorkItem} command
  * can be specified.
  * @param context The request context.
  * @return The response page.
  * @exception ClientException if an error occurred
  */
  public ResponsePage forceRestartWorkItemResponse(RequestContext context)
                      throws ClientException
  {
    WorkItem workItem = context.getWorkItem();

    if (context.getRequestParameter("submitData", null) == null)
      return new ResponsePage(context, "/forms/ForceRestart.jsp");

    return super.forceRestartWorkItemResponse(context);
  }

  /**
  * @deprecated - this function will be redesigned in a later release
  */
  public ResponsePage getFilterAssistantResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/FilterAssistantViewer.jsp");
  }

  /**
  * @deprecated - this function will be redesigned in a later release
  */
  public ResponsePage getSortCriteriaAssistantResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/SortCriteriaAssistantViewer.jsp");
  }

  /**
  * @deprecated - this function will be redesigned in a later release
  */
  public ResponsePage getUserInfoResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/UserInfoViewer.jsp");
  }

  /**
  * @deprecated - this function will be redesigned in a later release
  */
  public ResponsePage getSystemInfoResponse(RequestContext context)
                      throws ClientException
  {
    return new ResponsePage(context, "/forms/SystemInfoViewer.jsp");
  }

}
